// This file is part of the program FRYSK.
//
// Copyright 2005, 2007, 2008, Red Hat Inc.
//
// FRYSK is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// FRYSK is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with FRYSK.  If not, see <http://www.gnu.org/licenses/>.
// 
// In addition, as a special exception, Red Hat, Inc. gives You the
// additional right to link the code of FRYSK with code not covered
// under the GNU General Public License ("Non-GPL Code") and to
// distribute linked combinations including the two, subject to the
// limitations in this paragraph. Non-GPL Code permitted under this
// exception must only link to the code of FRYSK through those well
// defined interfaces identified in the file named EXCEPTION found in
// the source code files (the "Approved Interfaces"). The files of
// Non-GPL Code may instantiate templates or use macros or inline
// functions from the Approved Interfaces without causing the
// resulting work to be covered by the GNU General Public
// License. Only Red Hat, Inc. may make changes or additions to the
// list of Approved Interfaces. You must obey the GNU General Public
// License in all respects for all of the FRYSK code and other code
// used in conjunction with FRYSK except the Non-GPL Code covered by
// this exception. If you modify this file, you may extend this
// exception to your version of the file, but you are not obligated to
// do so. If you do not wish to provide this exception without
// modification, you must delete this exception statement from your
// version and license this file solely under the GPL without
// exception.

#include <signal.h>
#include <sys/time.h>
#include <errno.h>
#include <unistd.h>

#include "jni.hxx"

#include "jnixx/exceptions.hxx"

using namespace java::lang;
using namespace frysk::sys;

struct timeval
timeval(jlong milliseconds) {
  struct timeval val;
  val.tv_sec = milliseconds / 1000;
  val.tv_usec = (milliseconds % 1000) * 1000;
  return val;
}

void
setItimer(jnixx::env env, int which, jlong interval, jlong value) {
  struct itimerval itimer;
  itimer.it_interval = timeval (interval);
  itimer.it_value = timeval (value);
  errno = 0;
  if (::setitimer (which, &itimer, NULL) < 0)
    errnoException(env, errno, "setitimer");
}

Signal
Itimer::real(jnixx::env env, jlong interval, jlong value) {
  ::setItimer(env, ITIMER_REAL, interval, value);
  return Signal::GetALRM(env);
}

jint
Itimer::sleep(jnixx::env env, jint seconds) {
  return ::sleep(seconds);
}
